#!/usr/bin/env python
#
# Simple script to add user quotas to a group of users 
# selected by their group
#
# Copyright (C) 2009-2011 Simone Piccardi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

#
# Thanks to I.P.S.S.A.R.C.T. Elena Cornaro Jesolo Lido - Venezia
# for financing the development of this script
#

import getopt, re
from os import *
from sys import *
import ldap
import ldap.modlist as modlist

# help on line
def usage():
    print "Usage: setgroupquota.py [-h] group soft_block_limit hard_block_limit"
    print "  -h        print this help"
    print "  -d        debug run, just print commands and values"
    print "  -g grp    use ou=grp as base search for groups (instead of Groups)"
    print "  -p pwd    LDAP access pass (instead of trying read ldap.secret)"
    print
    print "ARGUMENTS:"
    print "  group is the group name which users will be put under the quotas"
    print "  soft_block_limit is the quota soft limit (in kilobytes)"
    print "  hard_block_limit is the quota hard limit (in kilobytes)"
    print " "


# option parsing
try:
    opts, args = getopt.getopt(argv[1:], 'hdp:g:')
    optval={}
    for opt, val in opts:
        if opt == "-h":
            usage()
            exit(0)
        else:
            optval[opt]=val
except getopt.GetoptError:
    print "Wrong options"
    usage()
    exit(2)

# check arguments: groupname, soft_limit and hard_limit must be present
if len(args) != 3:
    print "Wrong argument number"
    print
    usage()
    exit(0)

# Get LDAP parameters from standard files
flags  = re.IGNORECASE|re.MULTILINE
try:
    regexp = r'^\s*base\s*(.*)'
    regx   = re.compile(regexp, flags)
    conf   = file("/etc/ldap/ldap.conf").read()
    base   = regx.findall(conf)[0]
    regexp = r'^\s*uri\s*(.*)'
    regx   = re.compile(regexp, flags)
    uri    = regx.findall(conf)[0]
    if optval.has_key('-d'):
        print base
        print uri
except Exception,e:
    print "cannot read configuration from /etc/ldap/ldap.conf"
    print e

if  optval.has_key('-p'):
    password = optval['-p']
else:
    try:
        regexp = r'masterPw="(.*)"'
        regx   = re.compile(regexp, flags)
        conf   = file("/etc/smbldap-tools/smbldap_bind.conf").read()
        password = regx.findall(conf)[0]
    except Exception,e:
        print "cannot read password from /etc/smbldap-tools/smbldap_bind.conf"
        print e

# LDAP connection
try: 
    #l = ldap.open("127.0.0.1")
    l = ldap.initialize(uri)
    l.protocol_version = ldap.VERSION3
    username = "cn=admin,"+base
    l.simple_bind(username, password)
except ldap.LDAPError, e:
    print "Connection error", e


# LDAP search parameters
if optval.has_key('-g'):
    baseDN = "ou="+optval['-g']+","+base
else:
    baseDN = "ou=Groups,"+base

searchScope = ldap.SCOPE_ONELEVEL
searchFilter = "cn="+args[0]
retrieveAttributes = ["memberUid"]

# do LDAP search and set quota on results
try:
    r = l.search_s(baseDN, searchScope, searchFilter, retrieveAttributes)
    if r != []:
        for dn, entry in r:
            if optval.has_key('-d'):
                print dn
                print entry

            if entry.has_key('memberUid'):
                for user in entry['memberUid']:
                    cli='/usr/sbin/setquota '+user+' '+args[1]+' '+args[2]+' 0 0 /home/'
                    if optval.has_key('-d'):
                        print cli
                    else:
                        system(cli)
            else:
                print 'No members on group', args[0]

    else:
        print 'Cannot find group', args[0]

except ldap.LDAPError, e:
    print "Cannot perform search, group is", args[0]
    print "Search error", e

exit(0)

